/*
  KPF - Public fileserver for KDE

  Copyright 2001 Rik Hemsley (rikkus) <rik@kde.org>

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to
  deal in the Software without restriction, including without limitation the
  rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
  sell copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
  AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
  ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
  WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#ifndef KPF_UTILS_H
#define KPF_UTILS_H

#include <tqstringlist.h>
#include <tqstring.h>
#include <tqdatetime.h>
#include <tqfileinfo.h>

/**
 * Used to keep all parts of the kpf application out of the global namespace.
 */
namespace KPF
{
  /**
   * Safe version of TQMIN
   * @return minimum of a and b.
   */
  template<class T> T min(T a, T b) { return a < b ? a : b; }

  /**
   * Safe version of TQMAX
   * @return minimum of a and b.
   */
  template<class T> T max(T a, T b) { return b < a ? a : b; }

  /**
   * @return the current date and time as an HTTP/1.1 compliant date string.
   */
  TQString dateString();

  /**
   * @return the passed TQDateTime as an HTTP/1.1 compliant date string.
   */
  TQString dateString(const TQDateTime & dt);

  /**
   * Parse an HTTP/1.1 date to a TQDateTime.
   * @param ret the TQDateTime representation (result of parsing)
   * @return true if the date is an an acceptable format.
   */
  bool parseDate(const TQString &, TQDateTime & ret);

  /**
   * Parse an RFC 1123 format date to a TQDateTime. Usually called by
   * @ref parseDate.
   */
  bool parseDateRFC1123(const TQStringList &, TQDateTime &);

  /**
   * Parse an RFC 850 format date to a TQDateTime. Usually called by
   * @ref parseDate.
   */
  bool parseDateRFC850(const TQStringList &, TQDateTime &);

  /**
   * Parse an asctime(3) format date to a TQDateTime. Usually called by
   * @ref parseDate.
   */
  bool parseDateAscTime(const TQStringList &, TQDateTime &);

  /**
   * @return i18n(HTTP response message for code)
   */
  TQString translatedResponseName(uint code);

  /**
   * @return HTTP response message for code
   */
  TQString responseName(uint code);

  /**
   * @return the passed TQDateTime converted GMT, honouring daylight
   * saving time if necessary.
   */
  TQDateTime toGMT(const TQDateTime &);

  /**
   * Unquote hex quoted chars in string.
   */
  TQString unquote(const TQString &);

  /**
   * Quote naughty chars in %xx format.
   */
  TQString quote(const TQString &);

} // End namespace KPF

#endif
